# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..requests.tool_parameter import ToolParameterParams
from .raw_client import AsyncRawToolsClient, RawToolsClient
from .types.create_tool_request_endpoint_method import CreateToolRequestEndpointMethod
from .types.create_tool_request_execution_mode import CreateToolRequestExecutionMode
from .types.create_tool_request_type import CreateToolRequestType
from .types.tools_create_response import ToolsCreateResponse
from .types.tools_delete_response import ToolsDeleteResponse
from .types.tools_get_response import ToolsGetResponse
from .types.tools_list_response import ToolsListResponse
from .types.tools_update_response import ToolsUpdateResponse
from .types.update_tool_request_endpoint_method import UpdateToolRequestEndpointMethod
from .types.update_tool_request_execution_mode import UpdateToolRequestExecutionMode
from .types.update_tool_request_type import UpdateToolRequestType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawToolsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawToolsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawToolsClient
        """
        return self._raw_client

    def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ToolsListResponse:
        """
        Returns all custom tools for the organization.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list tools for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsListResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.tools.list(
            project="main",
        )
        """
        _response = self._raw_client.list(project=project, request_options=request_options)
        return _response.data

    def create(
        self,
        *,
        name: str,
        description: str,
        type: CreateToolRequestType,
        execution_mode: CreateToolRequestExecutionMode,
        project: typing.Optional[str] = None,
        parameters: typing.Optional[typing.Sequence[ToolParameterParams]] = OMIT,
        endpoint_method: typing.Optional[CreateToolRequestEndpointMethod] = OMIT,
        endpoint_url: typing.Optional[str] = OMIT,
        endpoint_headers: typing.Optional[typing.Dict[str, str]] = OMIT,
        endpoint_timeout_ms: typing.Optional[int] = OMIT,
        tool_call_output_timeout_ms: typing.Optional[int] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ToolsCreateResponse:
        """
        Creates a new tool in a project.

        Parameters
        ----------
        name : str
            The name of the tool. Must be snake_case and unique within the organization.

        description : str
            A description of what the tool does.

        type : CreateToolRequestType
            The type of tool.

        execution_mode : CreateToolRequestExecutionMode
            Mode of operation.

        project : typing.Optional[str]
            The name of the project to create the tool in.

        parameters : typing.Optional[typing.Sequence[ToolParameterParams]]
            Array of parameter definitions.
            For `custom_webhook` tools with POST method, each parameter must include a `location` field.
            For `custom_webhook` tools with GET method, `location` defaults to `"query_string"` if not specified.
            For `custom_websocket` and `built_in_transfer_to_phone_number` tools, `location` must not be specified.

        endpoint_method : typing.Optional[CreateToolRequestEndpointMethod]
            Required for webhook tools. HTTP method for the webhook endpoint.

        endpoint_url : typing.Optional[str]
            Required for webhook tools.

        endpoint_headers : typing.Optional[typing.Dict[str, str]]
            Optional headers for webhook tools.

        endpoint_timeout_ms : typing.Optional[int]
            Timeout for webhook tools.

        tool_call_output_timeout_ms : typing.Optional[int]
            Timeout for WebSocket tool responses.

        phone_number : typing.Optional[str]
            The E.164 formatted phone number to transfer calls to. Required for built_in_transfer_to_phone_number tools.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsCreateResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.tools.create(
            project="main",
            name="book_appointment",
            description="Books an appointment in the calendar system",
            type="custom_webhook",
            execution_mode="sync",
            parameters=[
                {
                    "type": "string",
                    "name": "date",
                    "description": "The date for the appointment in YYYY-MM-DD format",
                    "is_required": True,
                    "location": "request_body",
                },
                {
                    "type": "string",
                    "name": "time",
                    "description": "The time for the appointment in HH:MM format",
                    "is_required": True,
                    "location": "request_body",
                },
            ],
            endpoint_method="POST",
            endpoint_url="https://api.example.com/book-appointment",
            endpoint_headers={
                "Authorization": "Bearer token123",
                "Content-Type": "application/json",
            },
            endpoint_timeout_ms=5000,
        )
        """
        _response = self._raw_client.create(
            name=name,
            description=description,
            type=type,
            execution_mode=execution_mode,
            project=project,
            parameters=parameters,
            endpoint_method=endpoint_method,
            endpoint_url=endpoint_url,
            endpoint_headers=endpoint_headers,
            endpoint_timeout_ms=endpoint_timeout_ms,
            tool_call_output_timeout_ms=tool_call_output_timeout_ms,
            phone_number=phone_number,
            request_options=request_options,
        )
        return _response.data

    def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ToolsGetResponse:
        """
        Returns a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to get.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsGetResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.tools.get(
            name_or_id="nameOrId",
            project="main",
        )
        """
        _response = self._raw_client.get(name_or_id, project=project, request_options=request_options)
        return _response.data

    def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ToolsDeleteResponse:
        """
        Deletes a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to delete.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsDeleteResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.tools.delete(
            name_or_id="nameOrId",
            project="main",
        )
        """
        _response = self._raw_client.delete(name_or_id, project=project, request_options=request_options)
        return _response.data

    def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        type: typing.Optional[UpdateToolRequestType] = OMIT,
        execution_mode: typing.Optional[UpdateToolRequestExecutionMode] = OMIT,
        parameters: typing.Optional[typing.Sequence[ToolParameterParams]] = OMIT,
        endpoint_method: typing.Optional[UpdateToolRequestEndpointMethod] = OMIT,
        endpoint_url: typing.Optional[str] = OMIT,
        endpoint_headers: typing.Optional[typing.Dict[str, str]] = OMIT,
        endpoint_timeout_ms: typing.Optional[int] = OMIT,
        tool_call_output_timeout_ms: typing.Optional[int] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ToolsUpdateResponse:
        """
        Updates a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to update.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            The name of the tool. Must be snake_case and unique within the organization.

        description : typing.Optional[str]
            A description of what the tool does.

        type : typing.Optional[UpdateToolRequestType]
            The type of tool.

        execution_mode : typing.Optional[UpdateToolRequestExecutionMode]
            Mode of operation.

        parameters : typing.Optional[typing.Sequence[ToolParameterParams]]
            Array of parameter definitions.
            When updating `type` or `endpoint_method`, all parameters must include explicit `location` values.
            For `custom_webhook` tools: `location` is required for POST, defaults to `"query_string"` for GET.
            For `custom_websocket` and `built_in_transfer_to_phone_number` tools: `location` must not be specified.

        endpoint_method : typing.Optional[UpdateToolRequestEndpointMethod]
            HTTP method for webhook tools. When changing this value, all parameters must include explicit `location` values.

        endpoint_url : typing.Optional[str]

        endpoint_headers : typing.Optional[typing.Dict[str, str]]

        endpoint_timeout_ms : typing.Optional[int]

        tool_call_output_timeout_ms : typing.Optional[int]

        phone_number : typing.Optional[str]
            The E.164 formatted phone number to transfer calls to. Required for built_in_transfer_to_phone_number tools.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsUpdateResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.tools.update(
            name_or_id="nameOrId",
            project="main",
            description="Updated description for booking appointments with enhanced features",
            endpoint_headers={"Authorization": "Bearer updated_token456"},
            endpoint_timeout_ms=7000,
        )
        """
        _response = self._raw_client.update(
            name_or_id,
            project=project,
            name=name,
            description=description,
            type=type,
            execution_mode=execution_mode,
            parameters=parameters,
            endpoint_method=endpoint_method,
            endpoint_url=endpoint_url,
            endpoint_headers=endpoint_headers,
            endpoint_timeout_ms=endpoint_timeout_ms,
            tool_call_output_timeout_ms=tool_call_output_timeout_ms,
            phone_number=phone_number,
            request_options=request_options,
        )
        return _response.data


class AsyncToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawToolsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawToolsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawToolsClient
        """
        return self._raw_client

    async def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> ToolsListResponse:
        """
        Returns all custom tools for the organization.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list tools for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsListResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.list(
                project="main",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(project=project, request_options=request_options)
        return _response.data

    async def create(
        self,
        *,
        name: str,
        description: str,
        type: CreateToolRequestType,
        execution_mode: CreateToolRequestExecutionMode,
        project: typing.Optional[str] = None,
        parameters: typing.Optional[typing.Sequence[ToolParameterParams]] = OMIT,
        endpoint_method: typing.Optional[CreateToolRequestEndpointMethod] = OMIT,
        endpoint_url: typing.Optional[str] = OMIT,
        endpoint_headers: typing.Optional[typing.Dict[str, str]] = OMIT,
        endpoint_timeout_ms: typing.Optional[int] = OMIT,
        tool_call_output_timeout_ms: typing.Optional[int] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ToolsCreateResponse:
        """
        Creates a new tool in a project.

        Parameters
        ----------
        name : str
            The name of the tool. Must be snake_case and unique within the organization.

        description : str
            A description of what the tool does.

        type : CreateToolRequestType
            The type of tool.

        execution_mode : CreateToolRequestExecutionMode
            Mode of operation.

        project : typing.Optional[str]
            The name of the project to create the tool in.

        parameters : typing.Optional[typing.Sequence[ToolParameterParams]]
            Array of parameter definitions.
            For `custom_webhook` tools with POST method, each parameter must include a `location` field.
            For `custom_webhook` tools with GET method, `location` defaults to `"query_string"` if not specified.
            For `custom_websocket` and `built_in_transfer_to_phone_number` tools, `location` must not be specified.

        endpoint_method : typing.Optional[CreateToolRequestEndpointMethod]
            Required for webhook tools. HTTP method for the webhook endpoint.

        endpoint_url : typing.Optional[str]
            Required for webhook tools.

        endpoint_headers : typing.Optional[typing.Dict[str, str]]
            Optional headers for webhook tools.

        endpoint_timeout_ms : typing.Optional[int]
            Timeout for webhook tools.

        tool_call_output_timeout_ms : typing.Optional[int]
            Timeout for WebSocket tool responses.

        phone_number : typing.Optional[str]
            The E.164 formatted phone number to transfer calls to. Required for built_in_transfer_to_phone_number tools.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsCreateResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.create(
                project="main",
                name="book_appointment",
                description="Books an appointment in the calendar system",
                type="custom_webhook",
                execution_mode="sync",
                parameters=[
                    {
                        "type": "string",
                        "name": "date",
                        "description": "The date for the appointment in YYYY-MM-DD format",
                        "is_required": True,
                        "location": "request_body",
                    },
                    {
                        "type": "string",
                        "name": "time",
                        "description": "The time for the appointment in HH:MM format",
                        "is_required": True,
                        "location": "request_body",
                    },
                ],
                endpoint_method="POST",
                endpoint_url="https://api.example.com/book-appointment",
                endpoint_headers={
                    "Authorization": "Bearer token123",
                    "Content-Type": "application/json",
                },
                endpoint_timeout_ms=5000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            description=description,
            type=type,
            execution_mode=execution_mode,
            project=project,
            parameters=parameters,
            endpoint_method=endpoint_method,
            endpoint_url=endpoint_url,
            endpoint_headers=endpoint_headers,
            endpoint_timeout_ms=endpoint_timeout_ms,
            tool_call_output_timeout_ms=tool_call_output_timeout_ms,
            phone_number=phone_number,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ToolsGetResponse:
        """
        Returns a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to get.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsGetResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.get(
                name_or_id="nameOrId",
                project="main",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(name_or_id, project=project, request_options=request_options)
        return _response.data

    async def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ToolsDeleteResponse:
        """
        Deletes a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to delete.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsDeleteResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.delete(
                name_or_id="nameOrId",
                project="main",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(name_or_id, project=project, request_options=request_options)
        return _response.data

    async def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        type: typing.Optional[UpdateToolRequestType] = OMIT,
        execution_mode: typing.Optional[UpdateToolRequestExecutionMode] = OMIT,
        parameters: typing.Optional[typing.Sequence[ToolParameterParams]] = OMIT,
        endpoint_method: typing.Optional[UpdateToolRequestEndpointMethod] = OMIT,
        endpoint_url: typing.Optional[str] = OMIT,
        endpoint_headers: typing.Optional[typing.Dict[str, str]] = OMIT,
        endpoint_timeout_ms: typing.Optional[int] = OMIT,
        tool_call_output_timeout_ms: typing.Optional[int] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ToolsUpdateResponse:
        """
        Updates a tool by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the tool to update.

        project : typing.Optional[str]
            The name of the project containing the tool. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            The name of the tool. Must be snake_case and unique within the organization.

        description : typing.Optional[str]
            A description of what the tool does.

        type : typing.Optional[UpdateToolRequestType]
            The type of tool.

        execution_mode : typing.Optional[UpdateToolRequestExecutionMode]
            Mode of operation.

        parameters : typing.Optional[typing.Sequence[ToolParameterParams]]
            Array of parameter definitions.
            When updating `type` or `endpoint_method`, all parameters must include explicit `location` values.
            For `custom_webhook` tools: `location` is required for POST, defaults to `"query_string"` for GET.
            For `custom_websocket` and `built_in_transfer_to_phone_number` tools: `location` must not be specified.

        endpoint_method : typing.Optional[UpdateToolRequestEndpointMethod]
            HTTP method for webhook tools. When changing this value, all parameters must include explicit `location` values.

        endpoint_url : typing.Optional[str]

        endpoint_headers : typing.Optional[typing.Dict[str, str]]

        endpoint_timeout_ms : typing.Optional[int]

        tool_call_output_timeout_ms : typing.Optional[int]

        phone_number : typing.Optional[str]
            The E.164 formatted phone number to transfer calls to. Required for built_in_transfer_to_phone_number tools.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ToolsUpdateResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.tools.update(
                name_or_id="nameOrId",
                project="main",
                description="Updated description for booking appointments with enhanced features",
                endpoint_headers={"Authorization": "Bearer updated_token456"},
                endpoint_timeout_ms=7000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            name_or_id,
            project=project,
            name=name,
            description=description,
            type=type,
            execution_mode=execution_mode,
            parameters=parameters,
            endpoint_method=endpoint_method,
            endpoint_url=endpoint_url,
            endpoint_headers=endpoint_headers,
            endpoint_timeout_ms=endpoint_timeout_ms,
            tool_call_output_timeout_ms=tool_call_output_timeout_ms,
            phone_number=phone_number,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.not_found_error import NotFoundError
from ..requests.extraction_field import ExtractionFieldParams
from .types.extraction_schemas_create_response import ExtractionSchemasCreateResponse
from .types.extraction_schemas_delete_response import ExtractionSchemasDeleteResponse
from .types.extraction_schemas_get_response import ExtractionSchemasGetResponse
from .types.extraction_schemas_list_response import ExtractionSchemasListResponse
from .types.extraction_schemas_update_response import ExtractionSchemasUpdateResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawExtractionSchemasClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ExtractionSchemasListResponse]:
        """
        Returns all extraction schemas in a project.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list extraction schemas for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractionSchemasListResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "extraction_schemas",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasListResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        prompt: str,
        fields: typing.Sequence[ExtractionFieldParams],
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ExtractionSchemasCreateResponse]:
        """
        Creates a new extraction schema in a project.

        Parameters
        ----------
        name : str
            A name for the extraction schema.

        prompt : str
            Instructions for how to extract data from conversations.

        fields : typing.Sequence[ExtractionFieldParams]
            Array of field definitions.

        project : typing.Optional[str]
            The name of the project to create the extraction schema in.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractionSchemasCreateResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "extraction_schemas",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "project": project,
            },
            json={
                "name": name,
                "prompt": prompt,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields, annotation=typing.Sequence[ExtractionFieldParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasCreateResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ExtractionSchemasGetResponse]:
        """
        Returns an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to get.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractionSchemasGetResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"extraction_schemas/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasGetResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ExtractionSchemasDeleteResponse]:
        """
        Deletes an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to delete.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractionSchemasDeleteResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"extraction_schemas/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasDeleteResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        prompt: typing.Optional[str] = OMIT,
        fields: typing.Optional[typing.Sequence[ExtractionFieldParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ExtractionSchemasUpdateResponse]:
        """
        Updates an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to update.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            A name for the extraction schema.

        prompt : typing.Optional[str]
            Instructions for how to extract data from conversations.

        fields : typing.Optional[typing.Sequence[ExtractionFieldParams]]
            Array of field definitions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractionSchemasUpdateResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"extraction_schemas/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            params={
                "project": project,
            },
            json={
                "name": name,
                "prompt": prompt,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields, annotation=typing.Sequence[ExtractionFieldParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasUpdateResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawExtractionSchemasClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, project: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ExtractionSchemasListResponse]:
        """
        Returns all extraction schemas in a project.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list extraction schemas for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractionSchemasListResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "extraction_schemas",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasListResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        prompt: str,
        fields: typing.Sequence[ExtractionFieldParams],
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ExtractionSchemasCreateResponse]:
        """
        Creates a new extraction schema in a project.

        Parameters
        ----------
        name : str
            A name for the extraction schema.

        prompt : str
            Instructions for how to extract data from conversations.

        fields : typing.Sequence[ExtractionFieldParams]
            Array of field definitions.

        project : typing.Optional[str]
            The name of the project to create the extraction schema in.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractionSchemasCreateResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "extraction_schemas",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "project": project,
            },
            json={
                "name": name,
                "prompt": prompt,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields, annotation=typing.Sequence[ExtractionFieldParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasCreateResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasCreateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ExtractionSchemasGetResponse]:
        """
        Returns an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to get.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractionSchemasGetResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"extraction_schemas/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasGetResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ExtractionSchemasDeleteResponse]:
        """
        Deletes an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to delete.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractionSchemasDeleteResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"extraction_schemas/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            params={
                "project": project,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasDeleteResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasDeleteResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        name_or_id: str,
        *,
        project: typing.Optional[str] = None,
        name: typing.Optional[str] = OMIT,
        prompt: typing.Optional[str] = OMIT,
        fields: typing.Optional[typing.Sequence[ExtractionFieldParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ExtractionSchemasUpdateResponse]:
        """
        Updates an extraction schema by name or ID.

        Parameters
        ----------
        name_or_id : str
            The name or the ID of the extraction schema to update.

        project : typing.Optional[str]
            The name of the project containing the extraction schema. Only used when `nameOrId` is a name.

        name : typing.Optional[str]
            A name for the extraction schema.

        prompt : typing.Optional[str]
            Instructions for how to extract data from conversations.

        fields : typing.Optional[typing.Sequence[ExtractionFieldParams]]
            Array of field definitions.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractionSchemasUpdateResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"extraction_schemas/{jsonable_encoder(name_or_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="PATCH",
            params={
                "project": project,
            },
            json={
                "name": name,
                "prompt": prompt,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields, annotation=typing.Sequence[ExtractionFieldParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractionSchemasUpdateResponse,
                    parse_obj_as(
                        type_=ExtractionSchemasUpdateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class UpdateAgentRequestConfigurationEndpointParams(typing_extensions.TypedDict):
    """
    When not `null`, at the beginning of the conversation the agent will make a POST request to this endpoint when to get configuration options.
    """

    url: str
    """
    URL to call
    """

    headers: typing_extensions.NotRequired[typing.Dict[str, str]]
    """
    Object of key-value pairs.
    """

    timeout_ms: typing_extensions.NotRequired[int]
    """
    Timeout in milliseconds for the endpoint call.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

CreateToolRequestType = typing.Union[
    typing.Literal["custom_webhook", "custom_websocket", "built_in_transfer_to_phone_number"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ConversationsCancelResponse,
        ConversationsExtractDataResponse,
        ConversationsGetAnalysisResponse,
        ConversationsGetResponse,
        ConversationsListEvaluationsResponse,
        ConversationsListExtractionsResponse,
        ConversationsListResponse,
        ConversationsListResponseConversation,
        ConversationsListResponseConversations,
        ConversationsOutboundCallResponse,
        ConversationsSipOutboundCallResponse,
    )
    from .requests import (
        ConversationsCancelResponseParams,
        ConversationsExtractDataResponseParams,
        ConversationsGetAnalysisResponseParams,
        ConversationsGetResponseParams,
        ConversationsListEvaluationsResponseParams,
        ConversationsListExtractionsResponseParams,
        ConversationsListResponseConversationParams,
        ConversationsListResponseConversationsParams,
        ConversationsListResponseParams,
        ConversationsOutboundCallResponseParams,
        ConversationsSipOutboundCallResponseParams,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "ConversationsCancelResponse": ".types",
    "ConversationsCancelResponseParams": ".requests",
    "ConversationsExtractDataResponse": ".types",
    "ConversationsExtractDataResponseParams": ".requests",
    "ConversationsGetAnalysisResponse": ".types",
    "ConversationsGetAnalysisResponseParams": ".requests",
    "ConversationsGetResponse": ".types",
    "ConversationsGetResponseParams": ".requests",
    "ConversationsListEvaluationsResponse": ".types",
    "ConversationsListEvaluationsResponseParams": ".requests",
    "ConversationsListExtractionsResponse": ".types",
    "ConversationsListExtractionsResponseParams": ".requests",
    "ConversationsListResponse": ".types",
    "ConversationsListResponseConversation": ".types",
    "ConversationsListResponseConversationParams": ".requests",
    "ConversationsListResponseConversations": ".types",
    "ConversationsListResponseConversationsParams": ".requests",
    "ConversationsListResponseParams": ".requests",
    "ConversationsOutboundCallResponse": ".types",
    "ConversationsOutboundCallResponseParams": ".requests",
    "ConversationsSipOutboundCallResponse": ".types",
    "ConversationsSipOutboundCallResponseParams": ".requests",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ConversationsCancelResponse",
    "ConversationsCancelResponseParams",
    "ConversationsExtractDataResponse",
    "ConversationsExtractDataResponseParams",
    "ConversationsGetAnalysisResponse",
    "ConversationsGetAnalysisResponseParams",
    "ConversationsGetResponse",
    "ConversationsGetResponseParams",
    "ConversationsListEvaluationsResponse",
    "ConversationsListEvaluationsResponseParams",
    "ConversationsListExtractionsResponse",
    "ConversationsListExtractionsResponseParams",
    "ConversationsListResponse",
    "ConversationsListResponseConversation",
    "ConversationsListResponseConversationParams",
    "ConversationsListResponseConversations",
    "ConversationsListResponseConversationsParams",
    "ConversationsListResponseParams",
    "ConversationsOutboundCallResponse",
    "ConversationsOutboundCallResponseParams",
    "ConversationsSipOutboundCallResponse",
    "ConversationsSipOutboundCallResponseParams",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions
from .conversation_extraction_schema import ConversationExtractionSchemaParams


class ConversationExtractionParams(typing_extensions.TypedDict):
    id: str
    """
    The extraction ID.
    """

    conversation_id: typing_extensions.NotRequired[str]
    """
    The conversation ID.
    """

    schema: ConversationExtractionSchemaParams
    result: typing.Dict[str, typing.Optional[typing.Any]]
    """
    The extracted data.
    """

    error: typing.Optional[str]
    """
    Error message if extraction failed.
    """

    created_at: dt.datetime
    """
    When the extraction was created.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class TaskParams(typing_extensions.TypedDict):
    name: str
    """
    The name of the task.
    """

    description: str
    """
    The description of the task.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class ToolCallPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["tool_call"]
    tool_call_id: str
    """
    Unique ID for this tool call
    """

    tool_name: str
    """
    Name of the tool to execute
    """

    parameters: typing.Dict[str, typing.Optional[typing.Any]]
    """
    Parameters for tool execution
    """



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ExtractionSchemasCreateResponse,
        ExtractionSchemasDeleteResponse,
        ExtractionSchemasGetResponse,
        ExtractionSchemasListResponse,
        ExtractionSchemasUpdateResponse,
    )
    from .requests import (
        ExtractionSchemasCreateResponseParams,
        ExtractionSchemasDeleteResponseParams,
        ExtractionSchemasGetResponseParams,
        ExtractionSchemasListResponseParams,
        ExtractionSchemasUpdateResponseParams,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "ExtractionSchemasCreateResponse": ".types",
    "ExtractionSchemasCreateResponseParams": ".requests",
    "ExtractionSchemasDeleteResponse": ".types",
    "ExtractionSchemasDeleteResponseParams": ".requests",
    "ExtractionSchemasGetResponse": ".types",
    "ExtractionSchemasGetResponseParams": ".requests",
    "ExtractionSchemasListResponse": ".types",
    "ExtractionSchemasListResponseParams": ".requests",
    "ExtractionSchemasUpdateResponse": ".types",
    "ExtractionSchemasUpdateResponseParams": ".requests",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ExtractionSchemasCreateResponse",
    "ExtractionSchemasCreateResponseParams",
    "ExtractionSchemasDeleteResponse",
    "ExtractionSchemasDeleteResponseParams",
    "ExtractionSchemasGetResponse",
    "ExtractionSchemasGetResponseParams",
    "ExtractionSchemasListResponse",
    "ExtractionSchemasListResponseParams",
    "ExtractionSchemasUpdateResponse",
    "ExtractionSchemasUpdateResponseParams",
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ConversationEvaluationResultPromptParams(typing_extensions.TypedDict):
    """
    The evaluation prompt information.
    """

    id: str
    name: str



# This file was auto-generated by Fern from our API Definition.

import typing

ConfigPayloadOutputFormat = typing.Union[typing.Literal["pcm_44100", "pcm_16000", "pcm_8000", "mulaw_8000"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...requests.conversation_evaluation_result import ConversationEvaluationResultParams


class ConversationsListEvaluationsResponseParams(typing_extensions.TypedDict):
    evals: typing.Sequence[ConversationEvaluationResultParams]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ProjectsCreateEvalPromptResponse,
        ProjectsCreateResponse,
        ProjectsDeleteResponse,
        ProjectsGetResponse,
        ProjectsListEvalPromptsResponse,
        ProjectsListResponse,
        ProjectsUpdateResponse,
    )
    from .requests import (
        ProjectsCreateEvalPromptResponseParams,
        ProjectsCreateResponseParams,
        ProjectsDeleteResponseParams,
        ProjectsGetResponseParams,
        ProjectsListEvalPromptsResponseParams,
        ProjectsListResponseParams,
        ProjectsUpdateResponseParams,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "ProjectsCreateEvalPromptResponse": ".types",
    "ProjectsCreateEvalPromptResponseParams": ".requests",
    "ProjectsCreateResponse": ".types",
    "ProjectsCreateResponseParams": ".requests",
    "ProjectsDeleteResponse": ".types",
    "ProjectsDeleteResponseParams": ".requests",
    "ProjectsGetResponse": ".types",
    "ProjectsGetResponseParams": ".requests",
    "ProjectsListEvalPromptsResponse": ".types",
    "ProjectsListEvalPromptsResponseParams": ".requests",
    "ProjectsListResponse": ".types",
    "ProjectsListResponseParams": ".requests",
    "ProjectsUpdateResponse": ".types",
    "ProjectsUpdateResponseParams": ".requests",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ProjectsCreateEvalPromptResponse",
    "ProjectsCreateEvalPromptResponseParams",
    "ProjectsCreateResponse",
    "ProjectsCreateResponseParams",
    "ProjectsDeleteResponse",
    "ProjectsDeleteResponseParams",
    "ProjectsGetResponse",
    "ProjectsGetResponseParams",
    "ProjectsListEvalPromptsResponse",
    "ProjectsListEvalPromptsResponseParams",
    "ProjectsListResponse",
    "ProjectsListResponseParams",
    "ProjectsUpdateResponse",
    "ProjectsUpdateResponseParams",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class AgentConfigurationEndpointParams(typing_extensions.TypedDict):
    """
    When not `null`, the agent will call this endpoint to get configuration options.
    """

    url: str
    headers: typing.Dict[str, str]
    timeout_ms: int
    """
    The timeout for the configuration endpoint in milliseconds.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ConversationAgentParams(typing_extensions.TypedDict):
    """
    The agent associated with the conversation.
    """

    id: str
    """
    The ID of the agent.
    """

    name: str
    """
    The name of the agent.
    """

    is_deleted: bool
    """
    Whether the agent has been deleted.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class ConversationsExtractDataResponseParams(typing_extensions.TypedDict):
    result: typing.Dict[str, typing.Optional[typing.Any]]
    """
    The extracted data according to the schema.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..types.tool_endpoint_method import ToolEndpointMethod
from ..types.tool_execution_mode import ToolExecutionMode
from ..types.tool_type import ToolType
from .tool_parameter import ToolParameterParams
from .tool_project import ToolProjectParams


class ToolParams(typing_extensions.TypedDict):
    id: str
    """
    The ID of the tool.
    """

    name: str
    """
    The name of the tool.
    """

    description: str
    """
    Description of what the tool does.
    """

    project: ToolProjectParams
    type: ToolType
    """
    The type of tool.
    """

    execution_mode: ToolExecutionMode
    """
    Mode of operation - sync waits for response, async continues without waiting.
    """

    parameters: typing.Sequence[ToolParameterParams]
    """
    Array of parameter definitions for the tool.
    """

    endpoint_method: typing_extensions.NotRequired[ToolEndpointMethod]
    """
    HTTP method for webhook tools.
    """

    endpoint_url: typing_extensions.NotRequired[str]
    """
    URL for webhook tools.
    """

    endpoint_headers: typing_extensions.NotRequired[typing.Dict[str, str]]
    """
    Headers for webhook tools.
    """

    endpoint_timeout_ms: typing_extensions.NotRequired[int]
    """
    Timeout in milliseconds for webhook tools.
    """

    tool_call_output_timeout_ms: typing_extensions.NotRequired[int]
    """
    Timeout in milliseconds for WebSocket tool responses.
    """

    phone_number: typing_extensions.NotRequired[str]
    """
    The E.164 formatted phone number to transfer calls to. Required for built_in_transfer_to_phone_number tools.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class DtmfPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["dtmf"]
    digits: str
    """
    DTMF digits to play
    """



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .conversation_agent import ConversationAgent
from .conversation_background_noise import ConversationBackgroundNoise
from .conversation_ended_by import ConversationEndedBy
from .conversation_item import ConversationItem
from .conversation_project import ConversationProject


class Conversation(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The conversation ID.
    """

    agent: typing.Optional[ConversationAgent] = pydantic.Field(default=None)
    """
    The agent associated with the conversation.
    """

    workspace: str = pydantic.Field()
    """
    The organization/workspace name.
    """

    project: ConversationProject = pydantic.Field()
    """
    The project associated with the conversation.
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    External ID for conversation tracking.
    """

    model: str = pydantic.Field()
    """
    The STS model used.
    """

    welcome_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Welcome message played at start.
    """

    template_variables: typing.Dict[str, str] = pydantic.Field()
    """
    Template variables used in the conversation.
    """

    input_format: str = pydantic.Field()
    """
    Audio input format.
    """

    output_format: str = pydantic.Field()
    """
    Audio output format.
    """

    background_noise_level: float = pydantic.Field()
    """
    Background noise level used in the conversation.
    """

    background_noise: typing.Optional[ConversationBackgroundNoise] = pydantic.Field(default=None)
    """
    The background noise type used in the conversation.
    """

    live_transcript: str = pydantic.Field()
    """
    Live transcript of the conversation.
    """

    post_call_transcript: typing.Optional[str] = pydantic.Field(default=None)
    """
    Post-call processed transcript.
    """

    duration_ms: float = pydantic.Field()
    """
    Duration of the conversation in milliseconds.
    """

    audio_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Presigned URL to the conversation audio file.
    """

    started_at: dt.datetime = pydantic.Field()
    """
    When the conversation started.
    """

    ended_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When the conversation ended.
    """

    ended_by: typing.Optional[ConversationEndedBy] = pydantic.Field(default=None)
    """
    Who or what ended the conversation.
    """

    boosted_keywords: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    These words, or short phrases, are more accurately recognized by the model.
    """

    languages: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Array of ISO 639-1 language codes recognized by the model.
    """

    no_input_poke_sec: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of seconds of silence before a poke message is sent. `null` means the poke message is disabled.
    """

    no_input_poke_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    The message to send after the specified silence. Relevant only if `no_input_poke_sec` is not `null`.
    """

    no_input_end_conversation_sec: typing.Optional[int] = pydantic.Field(default=None)
    """
    Seconds of silence before the conversation is ended.
    """

    task_results: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Results from conversation evaluations and extractions.
    """

    items: typing.List[ConversationItem] = pydantic.Field()
    """
    Array of conversation items (turns).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class InputTextPayload(UniversalBaseModel):
    type: typing.Literal["input_text"] = "input_text"
    language: str = pydantic.Field()
    """
    Detected ISO 639-1 language code of user speech
    """

    text: str = pydantic.Field()
    """
    Transcribed user speech
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .config_payload_background_noise import ConfigPayloadBackgroundNoise
from .config_payload_input_format import ConfigPayloadInputFormat
from .config_payload_output_format import ConfigPayloadOutputFormat


class ConfigPayload(UniversalBaseModel):
    type: typing.Literal["config"] = "config"
    agent: typing.Optional[str] = pydantic.Field(default=None)
    """
    Agent name to use for conversation
    """

    project: typing.Optional[str] = pydantic.Field(default=None)
    """
    Project name
    """

    model: typing.Optional[typing.Literal["merritt"]] = pydantic.Field(default=None)
    """
    STS model to use
    """

    system_prompt: typing.Optional[str] = pydantic.Field(default=None)
    """
    System prompt for AI assistant
    """

    audio_speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    Audio playback speed
    """

    background_noise_level: typing.Optional[float] = pydantic.Field(default=None)
    """
    Background noise level for the conversation
    """

    background_noise: typing.Optional[ConfigPayloadBackgroundNoise] = pydantic.Field(default=None)
    """
    Background noise type for the conversation
    """

    welcome_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Message to play when conversation starts
    """

    voice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Voice ID to use for speech synthesis
    """

    input_format: typing.Optional[ConfigPayloadInputFormat] = pydantic.Field(default=None)
    """
    Audio input format
    """

    output_format: typing.Optional[ConfigPayloadOutputFormat] = pydantic.Field(default=None)
    """
    Audio output format
    """

    vad_prebuffer_duration_ms: typing.Optional[float] = pydantic.Field(default=None)
    """
    Voice activity detection prebuffer duration
    """

    vad_min_speech_duration_ms: typing.Optional[float] = pydantic.Field(default=None)
    """
    Minimum speech duration for VAD
    """

    vad_min_silence_duration_ms: typing.Optional[float] = pydantic.Field(default=None)
    """
    Minimum silence duration for VAD
    """

    vad_threshold: typing.Optional[float] = pydantic.Field(default=None)
    """
    Voice activity detection threshold
    """

    enable_documents_rag: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable document RAG
    """

    enable_transcripts_rag: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable transcript RAG
    """

    no_input_poke_sec: typing.Optional[float] = pydantic.Field(default=None)
    """
    Seconds of silence before poke message
    """

    no_input_poke_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    Poke message text
    """

    no_input_end_conversation_sec: typing.Optional[float] = pydantic.Field(default=None)
    """
    Seconds of silence before ending conversation
    """

    recognized_languages: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Array of ISO 639-1 language codes that the agent should be able to recognize
    """

    boosted_keywords: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Keywords to boost in speech recognition
    """

    tools: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Tools available to the assistant
    """

    template_variables: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Template variables for system prompt and welcome message
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class ToolCallInterruptedPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["tool_call_interrupted"]
    tool_call_id: str
    """
    ID of the interrupted tool call
    """

    tool_name: str
    """
    Name of the interrupted tool
    """

    error_message: str
    """
    Description of why the tool call was interrupted
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..types.language_code import LanguageCode
from .outbound_call_config_tools_item import OutboundCallConfigToolsItemParams


class OutboundCallConfigParams(typing_extensions.TypedDict):
    """
    When an `agent` is provided, these `config` options override the agent settings.
    """

    agent: typing_extensions.NotRequired[str]
    """
    The name of the agent to use for the call.
    """

    project: typing_extensions.NotRequired[str]
    """
    The name of the project to use for the call.
    """

    welcome_message: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Message to play when the conversation starts. Can contain template variables like `{{customer_name}}`.
    """

    system_prompt: typing_extensions.NotRequired[str]
    """
    Instructions for the conversation. Can contain template variables like `{{subject}}`.
    """

    template_variables: typing_extensions.NotRequired[typing.Dict[str, str]]
    """
    Variables that can be used in the welcome message and the system prompt.
    """

    voice_id: typing_extensions.NotRequired[str]
    """
    The voice ID to use for the agent.
    """

    no_input_poke_sec: typing_extensions.NotRequired[typing.Optional[int]]
    """
    Number of seconds of silence before sending a poke message. `null` disables the poke message.
    """

    no_input_poke_text: typing_extensions.NotRequired[str]
    """
    The message to send after the specified silence.
    """

    no_input_end_conversation_sec: typing_extensions.NotRequired[int]
    """
    Seconds of silence before ending the conversation.
    """

    languages: typing_extensions.NotRequired[typing.Sequence[LanguageCode]]
    """
    Array of ISO 639-1 language codes that the agent should be able to recognize
    """

    boosted_keywords: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    These words, or short phrases, will be more accurately recognized by the agent.
    """

    tools: typing_extensions.NotRequired[typing.Sequence[OutboundCallConfigToolsItemParams]]
    """
    Array of built-in or custom tool names to use.
    """



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .conversations_cancel_response import ConversationsCancelResponse
    from .conversations_extract_data_response import ConversationsExtractDataResponse
    from .conversations_get_analysis_response import ConversationsGetAnalysisResponse
    from .conversations_get_response import ConversationsGetResponse
    from .conversations_list_evaluations_response import ConversationsListEvaluationsResponse
    from .conversations_list_extractions_response import ConversationsListExtractionsResponse
    from .conversations_list_response import ConversationsListResponse
    from .conversations_list_response_conversation import ConversationsListResponseConversation
    from .conversations_list_response_conversations import ConversationsListResponseConversations
    from .conversations_outbound_call_response import ConversationsOutboundCallResponse
    from .conversations_sip_outbound_call_response import ConversationsSipOutboundCallResponse
_dynamic_imports: typing.Dict[str, str] = {
    "ConversationsCancelResponse": ".conversations_cancel_response",
    "ConversationsExtractDataResponse": ".conversations_extract_data_response",
    "ConversationsGetAnalysisResponse": ".conversations_get_analysis_response",
    "ConversationsGetResponse": ".conversations_get_response",
    "ConversationsListEvaluationsResponse": ".conversations_list_evaluations_response",
    "ConversationsListExtractionsResponse": ".conversations_list_extractions_response",
    "ConversationsListResponse": ".conversations_list_response",
    "ConversationsListResponseConversation": ".conversations_list_response_conversation",
    "ConversationsListResponseConversations": ".conversations_list_response_conversations",
    "ConversationsOutboundCallResponse": ".conversations_outbound_call_response",
    "ConversationsSipOutboundCallResponse": ".conversations_sip_outbound_call_response",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ConversationsCancelResponse",
    "ConversationsExtractDataResponse",
    "ConversationsGetAnalysisResponse",
    "ConversationsGetResponse",
    "ConversationsListEvaluationsResponse",
    "ConversationsListExtractionsResponse",
    "ConversationsListResponse",
    "ConversationsListResponseConversation",
    "ConversationsListResponseConversations",
    "ConversationsOutboundCallResponse",
    "ConversationsSipOutboundCallResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ConversationsSipOutboundCallResponseParams(typing_extensions.TypedDict):
    conversation_id: str
    """
    The ID of the created conversation.
    """



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        CreateToolRequestEndpointMethod,
        CreateToolRequestExecutionMode,
        CreateToolRequestType,
        ToolsCreateResponse,
        ToolsDeleteResponse,
        ToolsGetResponse,
        ToolsListResponse,
        ToolsUpdateResponse,
        UpdateToolRequestEndpointMethod,
        UpdateToolRequestExecutionMode,
        UpdateToolRequestType,
    )
    from .requests import (
        ToolsCreateResponseParams,
        ToolsDeleteResponseParams,
        ToolsGetResponseParams,
        ToolsListResponseParams,
        ToolsUpdateResponseParams,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "CreateToolRequestEndpointMethod": ".types",
    "CreateToolRequestExecutionMode": ".types",
    "CreateToolRequestType": ".types",
    "ToolsCreateResponse": ".types",
    "ToolsCreateResponseParams": ".requests",
    "ToolsDeleteResponse": ".types",
    "ToolsDeleteResponseParams": ".requests",
    "ToolsGetResponse": ".types",
    "ToolsGetResponseParams": ".requests",
    "ToolsListResponse": ".types",
    "ToolsListResponseParams": ".requests",
    "ToolsUpdateResponse": ".types",
    "ToolsUpdateResponseParams": ".requests",
    "UpdateToolRequestEndpointMethod": ".types",
    "UpdateToolRequestExecutionMode": ".types",
    "UpdateToolRequestType": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateToolRequestEndpointMethod",
    "CreateToolRequestExecutionMode",
    "CreateToolRequestType",
    "ToolsCreateResponse",
    "ToolsCreateResponseParams",
    "ToolsDeleteResponse",
    "ToolsDeleteResponseParams",
    "ToolsGetResponse",
    "ToolsGetResponseParams",
    "ToolsListResponse",
    "ToolsListResponseParams",
    "ToolsUpdateResponse",
    "ToolsUpdateResponseParams",
    "UpdateToolRequestEndpointMethod",
    "UpdateToolRequestExecutionMode",
    "UpdateToolRequestType",
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...requests.project import ProjectParams


class ProjectsGetResponseParams(typing_extensions.TypedDict):
    project: ProjectParams



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class AudioChunkResponsePayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["audio_chunk"]
    audio: str
    """
    Base64-encoded AI audio data
    """

    text: str
    """
    Text corresponding to audio chunk
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class CreateAgentRequestTemplateVariablesValueParams(typing_extensions.TypedDict):
    default_value: typing.Optional[str]
    is_boosted_keyword: typing_extensions.NotRequired[bool]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .error_payload_error import ErrorPayloadErrorParams


class ErrorPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["error"]
    error: ErrorPayloadErrorParams
    param_errors: typing_extensions.NotRequired[typing.Dict[str, str]]
    """
    Parameter-specific validation errors
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...requests.voice import VoiceParams


class VoicesListResponseParams(typing_extensions.TypedDict):
    voices: typing.Sequence[VoiceParams]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .validation_error_error import ValidationErrorErrorParams


class ValidationErrorParams(typing_extensions.TypedDict):
    error: ValidationErrorErrorParams
    param_errors: typing.Dict[str, str]
    """
    Parameter-specific validation errors
    """



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AgentsCreateResponse,
        AgentsDeleteResponse,
        AgentsGetResponse,
        AgentsListResponse,
        AgentsUpdateResponse,
        AgentsUpsertResponse,
        UpdateAgentRequestAudioFormat,
        UpdateAgentRequestBackgroundNoise,
        UpdateAgentRequestConfigurationEndpoint,
        UpdateAgentRequestTemplateVariablesValue,
        UpdateAgentRequestToolsItem,
    )
    from .requests import (
        AgentsCreateResponseParams,
        AgentsDeleteResponseParams,
        AgentsGetResponseParams,
        AgentsListResponseParams,
        AgentsUpdateResponseParams,
        AgentsUpsertResponseParams,
        UpdateAgentRequestConfigurationEndpointParams,
        UpdateAgentRequestTemplateVariablesValueParams,
        UpdateAgentRequestToolsItemParams,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "AgentsCreateResponse": ".types",
    "AgentsCreateResponseParams": ".requests",
    "AgentsDeleteResponse": ".types",
    "AgentsDeleteResponseParams": ".requests",
    "AgentsGetResponse": ".types",
    "AgentsGetResponseParams": ".requests",
    "AgentsListResponse": ".types",
    "AgentsListResponseParams": ".requests",
    "AgentsUpdateResponse": ".types",
    "AgentsUpdateResponseParams": ".requests",
    "AgentsUpsertResponse": ".types",
    "AgentsUpsertResponseParams": ".requests",
    "UpdateAgentRequestAudioFormat": ".types",
    "UpdateAgentRequestBackgroundNoise": ".types",
    "UpdateAgentRequestConfigurationEndpoint": ".types",
    "UpdateAgentRequestConfigurationEndpointParams": ".requests",
    "UpdateAgentRequestTemplateVariablesValue": ".types",
    "UpdateAgentRequestTemplateVariablesValueParams": ".requests",
    "UpdateAgentRequestToolsItem": ".types",
    "UpdateAgentRequestToolsItemParams": ".requests",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AgentsCreateResponse",
    "AgentsCreateResponseParams",
    "AgentsDeleteResponse",
    "AgentsDeleteResponseParams",
    "AgentsGetResponse",
    "AgentsGetResponseParams",
    "AgentsListResponse",
    "AgentsListResponseParams",
    "AgentsUpdateResponse",
    "AgentsUpdateResponseParams",
    "AgentsUpsertResponse",
    "AgentsUpsertResponseParams",
    "UpdateAgentRequestAudioFormat",
    "UpdateAgentRequestBackgroundNoise",
    "UpdateAgentRequestConfigurationEndpoint",
    "UpdateAgentRequestConfigurationEndpointParams",
    "UpdateAgentRequestTemplateVariablesValue",
    "UpdateAgentRequestTemplateVariablesValueParams",
    "UpdateAgentRequestToolsItem",
    "UpdateAgentRequestToolsItemParams",
]



# This file was auto-generated by Fern from our API Definition.

import json
import typing
from json.decoder import JSONDecodeError

import websockets
import websockets.sync.connection as websockets_sync_connection
from ..core.events import EventEmitterMixin, EventType
from ..core.pydantic_utilities import parse_obj_as
from ..types.assistant_chose_not_to_respond_payload import AssistantChoseNotToRespondPayload
from ..types.assistant_ended_conversation_payload import AssistantEndedConversationPayload
from ..types.audio_chunk_payload import AudioChunkPayload
from ..types.config_payload import ConfigPayload
from ..types.conversation_created_payload import ConversationCreatedPayload
from ..types.dtmf_payload import DtmfPayload
from ..types.error_payload import ErrorPayload
from ..types.input_cancelled_payload import InputCancelledPayload
from ..types.input_text_payload import InputTextPayload
from ..types.is_user_speaking_payload import IsUserSpeakingPayload
from ..types.ready_to_start_conversation_payload import ReadyToStartConversationPayload
from ..types.set_external_id_payload import SetExternalIdPayload
from ..types.set_twilio_call_sid_payload import SetTwilioCallSidPayload
from ..types.tool_call_interrupted_payload import ToolCallInterruptedPayload
from ..types.tool_call_output_payload import ToolCallOutputPayload
from ..types.tool_call_output_processed_payload import ToolCallOutputProcessedPayload
from ..types.tool_call_payload import ToolCallPayload
from ..types.update_system_prompt_payload import UpdateSystemPromptPayload
from ..types.user_finished_speaking_payload import UserFinishedSpeakingPayload
from ..types.user_started_speaking_payload import UserStartedSpeakingPayload

try:
    from websockets.legacy.client import WebSocketClientProtocol  # type: ignore
except ImportError:
    from websockets import WebSocketClientProtocol  # type: ignore

ConversationsSocketClientResponse = typing.Union[
    ReadyToStartConversationPayload,
    ConversationCreatedPayload,
    InputTextPayload,
    InputCancelledPayload,
    IsUserSpeakingPayload,
    UserStartedSpeakingPayload,
    UserFinishedSpeakingPayload,
    DtmfPayload,
    ToolCallPayload,
    ToolCallOutputProcessedPayload,
    ToolCallInterruptedPayload,
    AssistantChoseNotToRespondPayload,
    AssistantEndedConversationPayload,
    ErrorPayload,
]


class AsyncConversationsSocketClient(EventEmitterMixin):
    def __init__(self, *, websocket: WebSocketClientProtocol):
        super().__init__()
        self._websocket = websocket

    async def __aiter__(self):
        async for message in self._websocket:
            yield parse_obj_as(ConversationsSocketClientResponse, json.loads(message))  # type: ignore

    async def start_listening(self):
        """
        Start listening for messages on the websocket connection.

        Emits events in the following order:
        - EventType.OPEN when connection is established
        - EventType.MESSAGE for each message received
        - EventType.ERROR if an error occurs
        - EventType.CLOSE when connection is closed
        """
        await self._emit_async(EventType.OPEN, None)
        try:
            async for raw_message in self._websocket:
                json_data = json.loads(raw_message)
                parsed = parse_obj_as(ConversationsSocketClientResponse, json_data)  # type: ignore
                await self._emit_async(EventType.MESSAGE, parsed)
        except (websockets.WebSocketException, JSONDecodeError) as exc:
            await self._emit_async(EventType.ERROR, exc)
        finally:
            await self._emit_async(EventType.CLOSE, None)

    async def send_audio_chunk(self, message: AudioChunkPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a AudioChunkPayload.
        """
        await self._send_model(message)

    async def send_config(self, message: ConfigPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a ConfigPayload.
        """
        await self._send_model(message)

    async def send_update_system_prompt(self, message: UpdateSystemPromptPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a UpdateSystemPromptPayload.
        """
        await self._send_model(message)

    async def send_set_external_id(self, message: SetExternalIdPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a SetExternalIdPayload.
        """
        await self._send_model(message)

    async def send_set_twilio_call_sid(self, message: SetTwilioCallSidPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a SetTwilioCallSidPayload.
        """
        await self._send_model(message)

    async def send_tool_call_output(self, message: ToolCallOutputPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a ToolCallOutputPayload.
        """
        await self._send_model(message)

    async def recv(self) -> ConversationsSocketClientResponse:
        """
        Receive a message from the websocket connection.
        """
        data = await self._websocket.recv()
        json_data = json.loads(data)
        return parse_obj_as(ConversationsSocketClientResponse, json_data)  # type: ignore

    async def _send(self, data: typing.Any) -> None:
        """
        Send a message to the websocket connection.
        """
        if isinstance(data, dict):
            data = json.dumps(data)
        await self._websocket.send(data)

    async def _send_model(self, data: typing.Any) -> None:
        """
        Send a Pydantic model to the websocket connection.
        """
        await self._send(data.dict())


class ConversationsSocketClient(EventEmitterMixin):
    def __init__(self, *, websocket: websockets_sync_connection.Connection):
        super().__init__()
        self._websocket = websocket

    def __iter__(self):
        for message in self._websocket:
            yield parse_obj_as(ConversationsSocketClientResponse, json.loads(message))  # type: ignore

    def start_listening(self):
        """
        Start listening for messages on the websocket connection.

        Emits events in the following order:
        - EventType.OPEN when connection is established
        - EventType.MESSAGE for each message received
        - EventType.ERROR if an error occurs
        - EventType.CLOSE when connection is closed
        """
        self._emit(EventType.OPEN, None)
        try:
            for raw_message in self._websocket:
                json_data = json.loads(raw_message)
                parsed = parse_obj_as(ConversationsSocketClientResponse, json_data)  # type: ignore
                self._emit(EventType.MESSAGE, parsed)
        except (websockets.WebSocketException, JSONDecodeError) as exc:
            self._emit(EventType.ERROR, exc)
        finally:
            self._emit(EventType.CLOSE, None)

    def send_audio_chunk(self, message: AudioChunkPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a AudioChunkPayload.
        """
        self._send_model(message)

    def send_config(self, message: ConfigPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a ConfigPayload.
        """
        self._send_model(message)

    def send_update_system_prompt(self, message: UpdateSystemPromptPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a UpdateSystemPromptPayload.
        """
        self._send_model(message)

    def send_set_external_id(self, message: SetExternalIdPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a SetExternalIdPayload.
        """
        self._send_model(message)

    def send_set_twilio_call_sid(self, message: SetTwilioCallSidPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a SetTwilioCallSidPayload.
        """
        self._send_model(message)

    def send_tool_call_output(self, message: ToolCallOutputPayload) -> None:
        """
        Send a message to the websocket connection.
        The message will be sent as a ToolCallOutputPayload.
        """
        self._send_model(message)

    def recv(self) -> ConversationsSocketClientResponse:
        """
        Receive a message from the websocket connection.
        """
        data = self._websocket.recv()
        json_data = json.loads(data)
        return parse_obj_as(ConversationsSocketClientResponse, json_data)  # type: ignore

    def _send(self, data: typing.Any) -> None:
        """
        Send a message to the websocket connection.
        """
        if isinstance(data, dict):
            data = json.dumps(data)
        self._websocket.send(data)

    def _send_model(self, data: typing.Any) -> None:
        """
        Send a Pydantic model to the websocket connection.
        """
        self._send(data.dict())



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions


class AudioChunkPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["audio_chunk"]
    audio: str
    """
    Base64-encoded audio data (Int16Array for PCM, Uint8Array for mulaw)
    """

    iso_date_time: typing_extensions.NotRequired[dt.datetime]
    """
    ISO 8601 timestamp (required for first chunk only)
    """



# This file was auto-generated by Fern from our API Definition.

import datetime as dt

import typing_extensions
from ..types.conversation_evaluation_result_result import ConversationEvaluationResultResult
from .conversation_evaluation_result_prompt import ConversationEvaluationResultPromptParams


class ConversationEvaluationResultParams(typing_extensions.TypedDict):
    id: str
    """
    The evaluation ID.
    """

    result: ConversationEvaluationResultResult
    """
    The evaluation result.
    """

    prompt: ConversationEvaluationResultPromptParams
    """
    The evaluation prompt information.
    """

    created_at: dt.datetime
    """
    When the evaluation was created.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...requests.project import ProjectParams


class ProjectsListResponseParams(typing_extensions.TypedDict):
    projects: typing.Sequence[ProjectParams]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class InputTextPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["input_text"]
    language: str
    """
    Detected ISO 639-1 language code of user speech
    """

    text: str
    """
    Transcribed user speech
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ConversationEvalPromptParams(typing_extensions.TypedDict):
    id: str
    """
    The evaluation prompt ID.
    """

    name: str
    """
    The evaluation prompt name.
    """

    prompt: str
    """
    The evaluation prompt text.
    """



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions
from ..types.conversation_item_role import ConversationItemRole
from .conversation_item_tool_calls_item import ConversationItemToolCallsItemParams


class ConversationItemParams(typing_extensions.TypedDict):
    item_idx: int
    """
    Index of the item in the conversation.
    """

    role: ConversationItemRole
    """
    Who spoke in this turn.
    """

    live_transcript: str
    """
    Live transcript of this turn.
    """

    post_call_transcript: typing.Optional[str]
    """
    Post-call processed transcript.
    """

    duration_ms: float
    """
    Duration of this turn in milliseconds.
    """

    started_at: dt.datetime
    """
    When this turn started.
    """

    voice_id: typing_extensions.NotRequired[str]
    """
    Voice ID used (assistant only).
    """

    audio_speed: typing_extensions.NotRequired[float]
    """
    Audio speed used (assistant only).
    """

    system_prompt: typing_extensions.NotRequired[str]
    """
    System prompt used for this assistant turn.
    """

    tool_calls: typing_extensions.NotRequired[typing.Sequence[ConversationItemToolCallsItemParams]]
    """
    Tool calls made by the assistant.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...requests.agent import AgentParams


class AgentsListResponseParams(typing_extensions.TypedDict):
    agents: typing.Sequence[AgentParams]
